function ABlearn(iRun)

% <iRun>: the No. of the random run
% <Totalsketch>: allowed number of selected Gabors   

% load natural statistics
% load(sprintf('naturalStat_run%d',iRun));

% load the positive images
load(sprintf('../RandomSplits/split%d.mat',iRun),'posTrainImages');

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
maxNF = 80;
scale = 0.7;
norient = 15;
Upperbound = 6;
nF = 80;  % allowed number of selected features   

ax = 10; ay = ax;


% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ==================================================================
%  input images from a folder
% ==================================================================
posFolder = '../posAll';  % image input folder
nimage = length(posTrainImages);
Isize = zeros(nimage, 2);
I = cell(1,nimage);
for i = 1 : nimage
    I{i} = double(rgb2gray(imread([posFolder '\' posTrainImages{i}])));
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same

Sx = zeros(1, nimage) + sx;
Sy = zeros(1, nimage) + sy;

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('start filtering');
tic
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
allfiltered = applyfilterfftsame(I, allfilter);  % filter training images
disp(['filtering time: ' num2str(toc) ' seconds']);


% the following three lines are the only things that need to be added
mex Ctransform.c; 
Ctransform(nimage, norient, allfiltered, Sx, Sy, Upperbound);

hI = cell(size(allfiltered,1),size(allfiltered,2));
for i = 1:size(hI,1)
    for j = 1:size(hI,2)
        hI{i,j} = zeros(size(allfiltered{1,1}));
    end
end

varMap = 10*ones(size(allfiltered{1,1}));

disp('start learning');
tic

% compute map of orientation histogram
sub = max(floor(ax/5),floor(h/4));
CGraHist(nimage,norient,allfiltered,hI,varMap,Upperbound,sx,sy,ax,ay,h,sub);
towrite = varMap;
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
imwrite(towrite,'varMap.jpg');
varMapSaved = varMap;

% pursuit the least-variance positions
inhibited = zeros(sx,sy);
selectedMeanHist = zeros(nF,norient);
maxValue = max(varMap(:));
minVar = zeros(nF,1);
Mx = zeros(nF,1); My = zeros(nF,1);

q_logvolume = 5;

for k = 1:nF
    [minVar(k) ind] = min(varMap(:));
    if q_logvolume + (-1/2-log(sqrt(2*pi.*minVar(k)))) < 0.01 || minVar(k) > maxValue
        nF = k-1;
        break;
    end
    col = ceil(ind/sx); My(k) = col;
    row = ind - (col-1)*sx; Mx(k) = row;
    % calculate the mean gradient histogram
    hists = zeros(nimage,norient);
    for o = 1:norient
        for j = 1:nimage
            hists(j,o) = hI{j,o}(row,col);
        end
    end
    selectedMeanHist(k,:) = sum(hists,1)/nimage;
    % do inhibition
    left = max(1,col-ay);
    right = min(sy,col+ay);
    top = max(1,row-ax);
    bottom = min(sx,row+ax);
    inhibited(top:bottom,left:right) = 1;
    varMap(top:bottom,left:right) = maxValue + 1;
end
minVar = minVar(1:nF); Mx = Mx(1:nF); My = My(1:nF); selectedMeanHist = selectedMeanHist(1:nF,:); 
gain = q_logvolume + (-1/2-log(sqrt(2*pi.*minVar)));

disp(['learning time: ' num2str(toc) ' seconds']);


save(sprintf('model_run%d.mat',iRun),...
    'minVar','selectedMeanHist',...
    'inhibited','varMapSaved','towrite','Mx','My'...
    , 'ax','ay','sx','sy','gain','Upperbound','norient','nF','allfilter','h','maxNF','q_logvolume');


